#
# $Id: xi.msg.C,v 1.2.2.1 1999/02/08 08:09:00 yamasita Exp $
#

# FreeWnn is a network-extensible Kana-to-Kanji conversion system.
# This file is part of FreeWnn.
# 
# Copyright OMRON Corporation. 1987, 1988, 1989, 1990, 1991, 1992, 1999
# Copyright 1991 by Massachusetts Institute of Technology
#
# Author: OMRON SOFTWARE Co., Ltd. <freewnn@rd.kyoto.omronsoft.co.jp>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Emacs; see the file COPYING.  If not, write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Commentary:
#
# Change log:
#
# Last modified date: 8,Feb.1999
#
###
###	C message file for xim
###
0	Save was completed
1	CANDIDATES(SMALL)
2	CANDIDATES(LARGE)
3	You can't get candidates becuase candidates are too many.
4	Frequency
5	Evaluation value
6	Front vector
7	Evaluation value(LARGE)
8	Yomi
9	do you want remove
10	You can't remove a entry of hard-dictionary.
11	You can't remove a entry of read-only-dictionary.
12	{
13	} will be remove
14	You can't change because the frequency is read-only.
15	} will become unuse
16	  CANCEL  
17	JIS code input
18	Please input JIS code
19	Execute
20	Dictionary utility
21	Can't open default uumkey.
22	Can't expand.
23	Duplicate definition at key.
24	KUTEN input
25	Please input KUTEN
26	KU
27	Connection to jserver is closed.
28	Can't open a default uumrc.
29	Connection
30	Please input hostname
31	Connection is wrong
32	Connection failed
33	Connection was completed
34	  INSPECT   
35	DELE
36	 USE/UNUSE 
37	NEXT
38	PREV
39	  MOUSE   
40	   KEY    
41	  NEXT  
42	PREVIOUS
43	DIC ADD
44	DIC LIST
45	REGISTER
46	SEARCH
47	CHANGE PARA
48	SAVE
49	DIC INFO
50	CHANGE FUZ
51	 YES  
52	  NO  
53	 OK ? 
